"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.MacUpdater = undefined;

var _AppUpdater;

function _load_AppUpdater() {
    return _AppUpdater = require("./AppUpdater");
}

var _bluebirdLstC;

function _load_bluebirdLstC() {
    return _bluebirdLstC = _interopRequireDefault(require("bluebird-lst-c"));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class MacUpdater extends (_AppUpdater || _load_AppUpdater()).AppUpdater {
    constructor(options) {
        super(options);
        this.nativeUpdater = require("electron").autoUpdater;
        this.nativeUpdater.on("error", it => {
            if (this.logger != null) {
                this.logger.warn(it);
            }
            this.emit("error", it);
        });
        this.nativeUpdater.on("update-downloaded", () => {
            const version = this.versionInfo.version;
            if (this.logger != null) {
                this.logger.info(`New version ${ version } has been downloaded`);
            }
            this.emit("update-downloaded", this.versionInfo);
        });
    }
    onUpdateAvailable(versionInfo, fileInfo) {
        this.nativeUpdater.setFeedURL(versionInfo.releaseJsonUrl);
        super.onUpdateAvailable(versionInfo, fileInfo);
    }
    doDownloadUpdate(versionInfo, fileInfo) {
        this.nativeUpdater.checkForUpdates();
        return (_bluebirdLstC || _load_bluebirdLstC()).default.resolve();
    }
    quitAndInstall() {
        this.nativeUpdater.quitAndInstall();
    }
}
exports.MacUpdater = MacUpdater; //# sourceMappingURL=MacUpdater.js.map